/// \file 
/// \date 2010-04-28
/// \author Xavier Michelon, Toradex AG
///
/// This piece of software is released under the GNU Lesser General Public License
/// See http://www.gnu.org/licenses/lgpl.html for the licensing terms


#include "Oak.h"
#include <stdexcept>
#include <cstdio>
#include <cassert>


using namespace std;


//**********************************************************************************************************************
/// \brief Set the LED mode for the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[in] ledMode The new value for the LED mode. Refer to the device data sheet for the list of LED modes
/// \param[in] persistent If set to true, the LED mode will be stored in FLASH, otherwise it will be stored in RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool setLedMode(char* devicePath, unsigned char ledMode, bool persistent)
{
   assert(ledMode < 5); 
   unsigned char buffer[33] = { 0x00 /* report number */, 0x00 /* Set */, persistent ? 0x01 : 0x00, 0x01, 0x01, 0x00, 
                               ledMode};
   return Oak_Feature(devicePath, buffer, false);
}


//**********************************************************************************************************************
/// \brief Set the LED mode for the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[out] outLedMode Upon function exit, if the function returns true this variable hold the retrieved value of the 
/// LED mode
/// \param[in] persistent If set to true, the LED mode will be retrieved FLASH, otherwise it will be retrieved from RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool getLedMode(char* devicePath, unsigned char& outLedMode, bool persistent) 
{
   unsigned char buffer[33] = { 0x00 /* report number */, 0x01 /* Get */, persistent ? 0x01 : 0x00, 0x01, 0x01, 0x00 };
   if (!Oak_Feature(devicePath, buffer, true))
      return false;
   outLedMode = buffer[2];
   return true;
}


//**********************************************************************************************************************
/// \brief Get the report rate of the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[out] outReportRate Upon function exit, if the function returns true this variable hold the retrieved value 
/// of the report rate
/// \param[in] persistent If set to true, the report rate will be retrieved FLASH, otherwise it will be retrieved from 
/// RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool getReportRate(char* devicePath, unsigned short& outReportRate, bool persistent)
{
   unsigned char buffer[33] = { 0x00 /* report number */, 0x01 /* Get */, persistent ? 0x01 : 0x00, 0x02, 0x00, 0x00 };
   if (!Oak_Feature(devicePath, buffer, true))
      return false;
   outReportRate = (unsigned short)(buffer[2]) | ((unsigned short)(buffer[3]) << 8);
   return true;
}


//**********************************************************************************************************************
/// \brief Set the report rate for  the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[in] reportRate The new value for the report mode, in milliseconds
/// \param[in] persistent If set to true, the report rate will be stored in FLASH, otherwise it will be stored in RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool setReportRate(char* devicePath, unsigned short reportRate, bool persistent)
{
   unsigned char buffer[33] = { 0x00 /* report number */, 0x00 /* Set */, persistent ? 0x01 : 0x00, 0x02, 0x00, 0x00, 
      (unsigned char)(reportRate), (unsigned char)(reportRate >> 8)};
   return Oak_Feature(devicePath, buffer, false);
}


//**********************************************************************************************************************
/// \brief Get the sample rate of the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[out] outSampleRate Upon function exit, if the function returns true this variable hold the retrieved value 
/// of the sample rate
/// \param[in] persistent If set to true, the sample rate will be retrieved FLASH, otherwise it will be retrieved from 
/// RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool getSampleRate(char* devicePath, unsigned short& outSampleRate, bool persistent)
{
   unsigned char buffer[33] = { 0x00 /* report number */, 0x01 /* Get */, persistent ? 0x01 : 0x00, 0x02, 0x01, 0x00 };
   if (!Oak_Feature(devicePath, buffer, true))
      return false;
   outSampleRate = (unsigned short)(buffer[2]) | ((unsigned short)(buffer[3]) << 8);
   return true;
}


//**********************************************************************************************************************
/// \brief Set the sample rate for  the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[in] sampleRate The new value for the report mode, in milliseconds
/// \param[in] persistent If set to true, the sample rate will be stored in FLASH, otherwise it will be stored in RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool setSampleRate(char* devicePath, unsigned short sampleRate, bool persistent)
{
   unsigned char buffer[33] = { 0x00 /* report number */, 0x00 /* Set */, persistent ? 0x01 : 0x00, 0x02, 0x01, 0x00, 
      (unsigned char)(sampleRate), (unsigned char)(sampleRate >> 8)};
   return Oak_Feature(devicePath, buffer, false);
}


//**********************************************************************************************************************
/// \brief Get the report mode of the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[out] outReportMode Upon function exit, if the function returns true this variable hold the retrieved value 
/// of the report mode
/// \param[in] persistent If set to true, the report mode will be retrieved FLASH, otherwise it will be retrieved from 
/// RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool getReportMode(char* devicePath, unsigned char& outReportMode, bool persistent)
{
   unsigned char buffer[33] = { 0x00 /* report number */, 0x01 /* Get */, persistent ? 0x01 : 0x00, 0x01, 0x00, 0x00 };
   if (!Oak_Feature(devicePath, buffer, true))
      return false;
   outReportMode = buffer[2];
   return true;
}


//**********************************************************************************************************************
/// \brief Set the report mode for  the given sensor
///
/// \param[in] devicePath The path to the device
/// \param[in] reportMode The new value for the report mode, in milliseconds
/// \param[in] persistent If set to true, the report mode will be stored in FLASH, otherwise it will be stored in RAM
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool setReportMode(char* devicePath, unsigned char reportMode, bool persistent)
{
   assert(reportMode <= 2);
   unsigned char buffer[33] = { 0x00 /* report number */, 0x00 /* Set */, persistent ? 0x01 : 0x00, 0x01, 0x00, 0x00,
      reportMode };
   return Oak_Feature(devicePath, buffer, false);
}



//**********************************************************************************************************************
/// \brief Read the state of the Oak distance sensor. This function waits until the next Interrupt In report arrive
/// (delay depends on the sample rate, report mode and report rate), then parse the report, according the the 
/// format specified for the report in the device's datasheet, chapter 3.1.
///
/// \param[in] devicePath The path to the device
/// \param[out] outFrameNumber Upon function exit, if the function returns true this variable hold the frame number at
/// which the sampling was performed (the frame number is time stamp in milliseconds, internally coded on 11-bits, so
/// it wraps around every 2048 milliseconds.
/// \param[out] outPressure Upon function exit, if the function returns true this variable hold the pressure read by the
/// sensor
/// \param[out] outTemperature Upon function exit, if the function returns true this variable hold the temperature read
/// by the sensor
/// \return true if and only if the operation completed successfully
//**********************************************************************************************************************
bool readPSensorState(char* devicePath, unsigned short& outFrameNumber, unsigned short& outPressure,
                         unsigned short& outTemperature)
{
   unsigned char buffer[33];
   if (!Oak_GetInReport(devicePath, buffer, 33, 65535))
      return false;
   // buffer[0] is the report number (0), actual data starts at buffer[1]
   outFrameNumber = (unsigned short)buffer[1] | (((unsigned short)(buffer[2])) << 8);
   outPressure    = (unsigned short)buffer[3] | (((unsigned short)(buffer[4])) << 8);
   outTemperature = (unsigned short)buffer[5] | (((unsigned short)(buffer[6])) << 8);
   return true;
}


//**********************************************************************************************************************
/// \brief Program entry point
//**********************************************************************************************************************
int main()
{	
   try 
   {
      // Try to open a Oak P sensor (USB PID 0x2)
      tOakSensor sensor;
      if (!Oak_FindSensor(0x02, 00, 00, 00, 00, 00, 00, &sensor))
         throw std::runtime_error("No Oak P sensor found");

      // Display some informations about the sensor
      printf("Found a Oak P sensor\nUser Device Name: %s\nSerial Number: %s\n", sensor.UserDeviceName, sensor.SN) ;

      unsigned char ledMode;
      if (!getLedMode(sensor.DevicePath, ledMode, true))
         throw std::runtime_error("Error getting LED mode");
      
      // Play a bit with the LED mode
      printf("Current LED mode: %u\n", ledMode);
      ledMode = (ledMode + 1) % 5; // advance to the next LED mode (acceptable values are in the range [0..4])
      printf("New LED mode: %u\n", ledMode);
      // we set the LED mode to the next one
      if (!setLedMode(sensor.DevicePath, (ledMode + 1) % 5, true))
         throw std::runtime_error("Error setting LED mode");
         
      // Set the report mode to 'After Sampling'
      if (!setReportMode(sensor.DevicePath, 0, true))
         throw std::runtime_error("Error setting the report mode");

      // Set the sample rate to 100ms
      if (!setSampleRate(sensor.DevicePath, 100, true))
         throw std::runtime_error("Error setting sample rate");
      
            
      // now enter a loop where we read the device's state using the Interrupt reports arrive at the rate specified above
      while (GetAsyncKeyState(VK_ESCAPE) >= 0) // press the Escape key to end the program
      {
         unsigned short frameNumber(0), pressure(0), temperature(0);
         if (!readPSensorState(sensor.DevicePath, frameNumber, pressure, temperature))
            throw std::runtime_error("Error reading sensor state");
         // print the values, carefully taking care of the units specified in the datasheet.
         printf("Frame number : %5.3fs - Pressure: %6uPa - Temperature: %5.1fK\r", float(frameNumber) * 0.001f, pressure * 10, float(temperature)/ 10.0f );
      }
   } 
   catch (std::runtime_error const& e)
   {
      fprintf(stderr, "%s\n", e.what());
      return 1;
   }
   return 0;
}
